// Copyright 2012-2014, Philip Quinn and Sylvain Malacria. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
//  TSLogger.m
//  Touch Scroll Simulator
//

#import "TSLogger.h"
#import <math.h>

NSString *TSLoggerTestSession = @"Test";

static BOOL _CGFloatIsIntegral(CGFloat f)
{
    return (ceilf(f) == f && floorf(f) == f);
}

static NSString *_TSLogStringFromCGFloat(CGFloat f)
{
    if (_CGFloatIsIntegral(f))
        return [NSString stringWithFormat:@"%.0f", f];
    else
        return [NSString stringWithFormat:@"%.2f", f];
}


NSString *TSLogStringFromCGPoint(CGPoint pt)
{
    return [NSString stringWithFormat:@":P:%@:%@", 
                                        _TSLogStringFromCGFloat(pt.x), 
                                        _TSLogStringFromCGFloat(pt.y)];
}

NSString *TSLogStringFromTouches(NSSet *touches)
{
    NSMutableString *str = [[NSMutableString alloc] init];
    for (UITouch *touch in touches) {
        CGPoint loc = [touch locationInView:nil];
        CGPoint ploc = [touch previousLocationInView:nil];
        [str appendFormat:@":T:%d:%d:%f%@%@", 
         touch.phase, touch.tapCount, touch.timestamp,
         TSLogStringFromCGPoint(loc),
         TSLogStringFromCGPoint(ploc)];
    }    
    return str;
}

NSString *TSLogStringFromIVars(NSDictionary *iVars,CGFloat yOffset)
{
    NSMutableString *str = [[NSMutableString alloc] init];
    
    [str appendFormat:@":V"];
    NSString* fastscrollcount = [iVars objectForKey:@"_fastScrollCount"];
    [str appendFormat:@":%@",
     fastscrollcount];
    
    NSString* fastScrollEndtime = [iVars objectForKey:@"_fastScrollEndTime"];
    [str appendFormat:@":%@",
     fastScrollEndtime];
    
    NSString* fastScrollMultiplier = [iVars objectForKey:@"_fastScrollMultiplier"];
    [str appendFormat:@":%@",
     fastScrollMultiplier];
    
    NSString* fastScrollStartMultiplier = [iVars objectForKey:@"_fastScrollStartMultiplier"];
    [str appendFormat:@":%@",
     fastScrollStartMultiplier];
    
    NSString* verticalVelocity = [iVars objectForKey:@"_verticalVelocity"];
    [str appendFormat:@":%@",
     verticalVelocity];

    NSString* previousVerticalVelocity = [iVars objectForKey:@"_previousVerticalVelocity"];
    [str appendFormat:@":%@",
     previousVerticalVelocity];
    
    NSString* lastUpdateOffsetY = [iVars objectForKey:@"_lastUpdateOffsetY"];
    [str appendFormat:@":%@",
     lastUpdateOffsetY];
    
    [str appendFormat:@":%f",
     yOffset];
    
    return str;
}

@interface TSLogger ()
@property (strong) NSFileHandle *logFile;
@end

@implementation TSLogger
@synthesize logFile;

+(id)sharedLogger
{
    static dispatch_once_t onceToken = 0;
    __strong static id _sharedObject = nil;
    dispatch_once(&onceToken, ^{
        _sharedObject = [[self alloc] init];
    });
    return _sharedObject;
}

-(void)startSession:(NSString *)name
{
    NSArray *dirs = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *docDir = [dirs lastObject];
    
    NSString *path = [docDir stringByAppendingPathComponent:[NSString stringWithFormat:@"Session - %@.log", name]];
    int extender = 1;
    while ([[NSFileManager defaultManager] fileExistsAtPath:path]) {
        path = [docDir stringByAppendingPathComponent:[NSString stringWithFormat:@"Session - %@ %d.log", name, extender++]];
    }
    
    NSURL *url = [NSURL fileURLWithPath:path isDirectory:NO];
    if (self.logFile)
        [self endSession];
    [[NSFileManager defaultManager] createFileAtPath:path contents:nil attributes:nil];
    self.logFile = [NSFileHandle fileHandleForWritingToURL:url
                                                     error:NULL];
    [self writeLine:@"START"];
}

-(void)writeLine:(NSString*)line
{
    NSString *fullLine = [NSString stringWithFormat:@"%f:%@\n", 
                          [NSDate timeIntervalSinceReferenceDate],
                          line];
    //NSLog(@"%@", line);
    [self.logFile writeData:[fullLine dataUsingEncoding:NSUTF8StringEncoding]];
}

-(void)writeLineFormat:(NSString*)format, ...
{
    va_list args;
    va_start(args, format);
    NSString *line = [[NSString alloc] initWithFormat:format arguments:args];
    [self writeLine:line];
}

-(void)endSession
{
    [self writeLine:@"END"];
    [self.logFile synchronizeFile];
    [self.logFile closeFile];
    self.logFile = nil;
}

@end
