// Copyright 2012-2014, Philip Quinn and Sylvain Malacria. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
//  TSMenuViewController.m
//  TouchScrollSimulator
//

#import "TSMenuViewController.h"
#import "TSRecordedScriptsController.h"
#import "TSScrollViewController.h"
#import "TSLogger.h"

// Segue identifiers
static NSString *kMenuSelectScriptSegue = @"selectScript";
static NSString *kMenuRecordScrollSegue = @"recordScroll";
static NSString *kMenuPlaybackScrollSegue = @"playbackScroll";
static NSString *kMenuTestScrollSegue = @"testScroll";

#define IS_ITEM(ip,s,r) (ip.section == s && ip.row == r)

@interface TSMenuViewController ()
// URL of the script file to use for playback
@property (strong) NSURL *selectedScriptURL;
@end

@implementation TSMenuViewController
@synthesize selectedScriptURL = _selectedScriptURL;
@synthesize selectedScriptLabel = _selectedScriptLabel;
@synthesize scriptNameTextField = _scriptNameTextField;

-(void)viewWillAppear:(BOOL)animated
{
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(_keyboardDidShow:) 
                                                 name:UIKeyboardDidShowNotification object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(_keyboardWillHide:)
                                                 name:UIKeyboardWillHideNotification object:nil];
    
    [self.navigationController setNavigationBarHidden:NO animated:YES];
    [self.tableView deselectRowAtIndexPath:[self.tableView indexPathForSelectedRow] animated:YES];
}

-(void)viewWillDisappear:(BOOL)animated
{
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}

-(void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender
{
    if ([segue.identifier isEqualToString:kMenuSelectScriptSegue]) {
        // Setup delegate to report back selection
        ((TSRecordedScriptsController*)segue.destinationViewController).delegate = self;
    } else if ([segue.identifier isEqualToString:kMenuRecordScrollSegue]) {
        // Setup recording
        NSString *label = self.scriptNameTextField.text;
        TSScrollViewController* viewController = (TSScrollViewController*)segue.destinationViewController;
        [viewController recordScriptWithLabel:label];
    } else if ([segue.identifier isEqualToString:kMenuPlaybackScrollSegue]) {
        // Setup playback
        [[TSLogger sharedLogger] startSession:self.selectedScriptLabel.text];
        TSScrollViewController* viewController = (TSScrollViewController*)segue.destinationViewController;
        [viewController playScriptAtURL:self.selectedScriptURL];
    } else if ([segue.identifier isEqualToString:kMenuTestScrollSegue]) {
        // Test view
        NSString* file = [[NSString alloc] initWithFormat:@"%@-%d",TSLoggerTestSession,(int)CFAbsoluteTimeGetCurrent()];
        [[TSLogger sharedLogger] startSession:file];
    }
    
    if ([segue.identifier isEqualToString:kMenuRecordScrollSegue] ||
        [segue.identifier isEqualToString:kMenuPlaybackScrollSegue] ||
        [segue.identifier isEqualToString:kMenuTestScrollSegue]) {
        TSScrollViewController* viewController = (TSScrollViewController*)segue.destinationViewController;
        viewController.shouldAutomaticallyReset = self.automaticallyReset.on;
        viewController.resetToTop = self.resetToTop.on;
        viewController.startAtBottom = self.startAtBottom.on;
    }
}

-(void)selectedRecordedScriptURL:(NSURL *)url
{
    // Don't deselect a script, even if one wasn't chosen
    if (url) {
        self.selectedScriptURL = url;
        self.selectedScriptLabel.text = [[url lastPathComponent] stringByDeletingPathExtension];
        [self.tableView reloadData];
    }
    [self.tableView deselectRowAtIndexPath:[self.tableView indexPathForSelectedRow] animated:YES];
}

-(NSIndexPath *)tableView:(UITableView *)tableView willSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    if (IS_ITEM(indexPath, 1, 0)) {
        // Recording label
        return nil;
    } else if (IS_ITEM(indexPath, 1, 1) && [self.scriptNameTextField.text isEqualToString:@""]) {
        // Start Recording
        UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"No Recording Label Set"
                                                        message:@"Data will not be recored." 
                                                       delegate:nil cancelButtonTitle:@"Continue" otherButtonTitles:nil];
        [alert show];
    } else if (IS_ITEM(indexPath, 2, 1) && !self.selectedScriptURL) {
        // Start Playback
        UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"No Script Selected"
                                                        message:@"A script must be selected before starting playback." 
                                                       delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
        [alert show];
        return nil;
    }
    return indexPath;
}

#pragma mark Keyboard Management

-(BOOL)textFieldShouldReturn:(UITextField*)textField
{
    [textField resignFirstResponder];
    return YES;
}

-(void)_keyboardDidShow:(NSNotification*)n
{
    CGSize keyboard = [[[n userInfo] objectForKey:UIKeyboardFrameEndUserInfoKey] CGRectValue].size;    
    
    UIEdgeInsets contentInsets = UIEdgeInsetsMake(0.f, 0.f, keyboard.height, 0.f);
    self.tableView.contentInset = contentInsets;
    self.tableView.scrollIndicatorInsets = contentInsets;
}

-(void)_keyboardWillHide:(NSNotification*)n
{
    self.tableView.contentInset = UIEdgeInsetsZero;
    self.tableView.scrollIndicatorInsets = UIEdgeInsetsZero;
}

@end
