// Copyright 2012-2014, Philip Quinn and Sylvain Malacria. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
//  TSSScrollView.m
//  TouchScrollSimulator
//

#import "TSScrollView.h"

#import <objc/runtime.h>

#import "PrivateAPI/UIScrollView-Static.h"
#import "PrivateAPI/UIScrollView-UIScrollViewInternal.h"

#import "TSLogger.h"
#import "TSLoggingResponder.h"

NSString *TSScrollViewStartedScrollingNotification = @"TSScrollViewStartedScrollingNotification";
NSString *TSScrollViewStoppedScrollingNotification = @"TSScrollViewStoppedSrcollingNotification";

@interface UIScrollView (PrivateAPI)
-(void)delayed:(UIGestureRecognizer*)gesture;
-(void)handlePan:(UIPanGestureRecognizer*)gesture;
- (void)_updatePanGesture;
- (void)_endPanWithEvent:(id)arg1;
- (void)_handleSwipe:(id)arg1;  // Never seems to be called...
@end

@interface TSScrollView ()
-(NSDictionary*)_ivars;
-(void)_init;
@end

@implementation TSScrollView
-(id)initWithFrame:(CGRect)frame
{
    if (self = [super initWithFrame:frame]) {
        [self _init];
    }
    return self;
}

-(id)initWithCoder:(NSCoder *)aDecoder
{
    self = [super initWithCoder:aDecoder];
    [self _init];
    return self;
}

-(void)_init
{
    // Find the gesture recogniser used to detect pans/swipes
    UIGestureRecognizer *panRecogniser = nil;
    for (UIGestureRecognizer *gs in self.gestureRecognizers) {
        if ([gs class] == NSClassFromString(@"UIScrollViewPanGestureRecognizer")) {
            panRecogniser = gs;
            break;
        }
    }
    
    // Swap its class for a subclass the logs the touches*:withEvent: messages
    // to the TSLogger.
    object_setClass(panRecogniser, TSLoggingScrollViewPanGestureRecognizer());
}

-(BOOL)touchesShouldBegin:(NSSet *)touches withEvent:(UIEvent *)event inContentView:(UIView *)view
{
    // Never forward touches to the content view
    return NO;
}

-(void)delayed:(UIGestureRecognizer*)gesture
{
    // Not sure when this is called...
    [super delayed:gesture];
}

#pragma mark Scroll Initiation
/* Called at the very start of a touch */
-(BOOL)_beginTrackingWithEvent:(UIEvent*)ev
{
    [[TSLogger sharedLogger] 
     writeLineFormat:@"SCROLLVIEW#TOUCH,T%@", TSLogStringFromTouches([ev allTouches])];
    [[TSLogger sharedLogger]
     writeLineFormat:@"SCROLLVIEW#TOUCH,P%@", TSLogStringFromCGPoint(self.contentOffset)];
    [[TSLogger sharedLogger]
     writeLineFormat:@"SCROLLVIEW#TOUCHIVARS%@", TSLogStringFromIVars([self _ivars],self.contentOffset.y)];
    [[NSNotificationCenter defaultCenter] postNotificationName:TSScrollViewStartedScrollingNotification object:self];
    return [super _beginTrackingWithEvent:ev];
}

/* Called at the start of a drag */
-(void)_resetScrollingWithUIEvent:(UIEvent*)ev
{
    [[TSLogger sharedLogger] 
     writeLineFormat:@"SCROLLVIEW#RESET,T%@", TSLogStringFromTouches([ev allTouches])];
    [[TSLogger sharedLogger]
     writeLineFormat:@"SCROLLVIEW#RESET,P%@", TSLogStringFromCGPoint(self.contentOffset)];
    [super _resetScrollingWithUIEvent:ev];
}

#pragma mark Scroll Dragging
/* Called for each move while dragging */
-(void)handlePan:(UIPanGestureRecognizer*)gesture
{
    NSMutableString *logLine = [[NSMutableString alloc] init];
    for (NSUInteger i = 0; i < [gesture numberOfTouches]; i++) {
        CGPoint pt = [gesture locationOfTouch:i inView:self];
        [logLine appendFormat:@"%@", TSLogStringFromCGPoint(pt)];
    }
    
    [[TSLogger sharedLogger] writeLineFormat:@"SCROLLVIEW#DRAG,T%@", logLine];
    [[TSLogger sharedLogger]
     writeLineFormat:@"SCROLLVIEW#DRAG,P%@", TSLogStringFromCGPoint(self.contentOffset)];

    [super handlePan:gesture];
}

/* Called for each move while dragging */
-(void)_updatePanGesture
{
    [[TSLogger sharedLogger]
     writeLineFormat:@"SCROLLVIEW#MOVE%@", TSLogStringFromCGPoint(self.contentOffset)];

    [super _updatePanGesture];
}

/* Called once at the start of the drag */
-(void)_scrollViewWillBeginDragging
{
    [[TSLogger sharedLogger]
     writeLineFormat:@"SCROLLVIEW#BEGINDRAG%@", TSLogStringFromCGPoint(self.contentOffset)];

    [super _scrollViewWillBeginDragging];
}

/* Called at the end of a pan following a drag */
-(void)_endPanWithEvent:(id)arg1
{
    [[TSLogger sharedLogger]
     writeLineFormat:@"SCROLLVIEW#ENDPAN%@", TSLogStringFromCGPoint(self.contentOffset)];

    [super _endPanWithEvent:arg1];
}

/* Called when scrolling stops after dragging
 * `decelerating` parameter indicates if a deceleration will follow. */
-(void)_scrollViewDidEndDraggingWithDeceleration:(BOOL)decelerating
{
    [[TSLogger sharedLogger]
     writeLineFormat:@"SCROLLVIEW#ENDDRAG:%@%@",
                     (decelerating ? @"YES" : @"NO"),
                     TSLogStringFromCGPoint(self.contentOffset)];
    [[TSLogger sharedLogger]
     writeLineFormat:@"SCROLLVIEW#UPIVARS%@", TSLogStringFromIVars([self _ivars],self.contentOffset.y)];
    if (!decelerating) {
        [[NSNotificationCenter defaultCenter] postNotificationName:TSScrollViewStoppedScrollingNotification object:self];
    }

    [super _scrollViewDidEndDraggingWithDeceleration:decelerating];
}

#pragma mark Smooth Scrolling

/* Called repeatedly to animate the smooth deceleration */
-(void)_smoothScroll:(CADisplayLink*)timer
{
    [[TSLogger sharedLogger]
     writeLineFormat:@"SCROLLVIEW#SMOOTH%@", TSLogStringFromCGPoint(self.contentOffset)];
    [super _smoothScroll:timer];
}

/* Called when scrolling finally stops after deceleration */
-(void)_stopScrollingNotify:(BOOL)arg1 dealloc:(BOOL)arg2 pin:(BOOL)arg3
{
    [[TSLogger sharedLogger]
     writeLineFormat:@"SCROLLVIEW#STOPSCROLL%@", TSLogStringFromCGPoint(self.contentOffset)];
    [super _stopScrollingNotify:arg1 dealloc:arg2 pin:arg3];
}

/* Called when scrolling finally stops after deceleration */
-(void)_stopScrollDecelerationNotify:(BOOL)arg1
{
    [[TSLogger sharedLogger]
     writeLineFormat:@"SCROLLVIEW#STOPDECEL%@", TSLogStringFromCGPoint(self.contentOffset)];
    [super _stopScrollDecelerationNotify:arg1];
}

/* Called at the end of scroll motion following deceleration */
-(void)_scrollViewDidEndDecelerating
{
    [[TSLogger sharedLogger]
     writeLineFormat:@"SCROLLVIEW#ENDDECEL%@", TSLogStringFromCGPoint(self.contentOffset)];
    NSLog(@"%.3f",self.contentOffset.y);
    [[NSNotificationCenter defaultCenter] postNotificationName:TSScrollViewStoppedScrollingNotification object:self];
    
    [super _scrollViewDidEndDecelerating];
}

#pragma mark Scroll Progress
/* Called whenever the scroll position changes */
-(void)_notifyDidScroll
{
    [[TSLogger sharedLogger]
     writeLineFormat:@"SCROLLVIEW#SCROLLED%@", TSLogStringFromCGPoint(self.contentOffset)];
    
    [[TSLogger sharedLogger]
     writeLineFormat:@"SCROLLVIEW#IVARS%@", TSLogStringFromIVars([self _ivars],self.contentOffset.y)];
    
    [super _notifyDidScroll];
}

#pragma mark Logging

-(id)valueForUndefinedKey:(NSString *)key
{
    return nil;
}

-(NSDictionary*)_ivars
{
    unsigned int nIvars = 0;
    Ivar *ivars = class_copyIvarList([UIScrollView class], &nIvars);
    if (!nIvars) {
        free(ivars);
        return nil;
    }
    
    NSMutableDictionary *dict = [[NSMutableDictionary alloc] init];
    for (int i = 0; i < nIvars; i++) {
        const char *name = ivar_getName(ivars[i]);
        NSString *key = [NSString stringWithCString:name encoding:NSASCIIStringEncoding];
        id value = [self valueForKey:key];
        if (value) {
            [dict setObject:value forKey:key];
        } else {
            [dict setObject:[NSNull null] forKey:key];
        }
    }
    free(ivars);
    return dict;
}

@end
